@@ -6,11 +6,13 @@ import asyncio
 import logging
 import sys
 import os
+import os.path
 import time
 import traceback
 import shutil
 import subprocess
 import warnings
+from typing import Dict, Type
 from socket import gethostname, getfqdn
 import attr
 from autobahn.asyncio.wamp import ApplicationRunner, ApplicationSession
@@ -26,7 +28,7 @@ except pkg_resources.DistributionNotFound:
     __version__ = "unknown"
 
 
-exports = {}
+exports: Dict[str, Type[ResourceEntry]] = {}
 reexec = False
 
 class ExporterError(Exception):
@@ -41,7 +43,7 @@ def log_subprocess_kernel_stack(logger, child):
     if child.poll() is not None:  # nothing to check if no longer running
         return
     try:
-        with open('/proc/{}/stack'.format(child.pid), 'r') as f:
+        with open(f'/proc/{child.pid}/stack', 'r') as f:
             stack = f.read()
             stack = stack.strip()
     except PermissionError:
@@ -63,7 +65,7 @@ class ResourceExport(ResourceEntry):
 
     def __attrs_post_init__(self):
         super().__attrs_post_init__()
-        self.logger = logging.getLogger("ResourceExport({})".format(self.cls))
+        self.logger = logging.getLogger(f"ResourceExport({self.cls})")
         # move the params to local_params
         self.local_params = self.params.copy()
         for key in self.local_params:
@@ -163,13 +165,13 @@ class ResourceExport(ResourceEntry):
 
     def acquire(self, *args, **kwargs):  # pylint: disable=arguments-differ
         if self.broken:
-            raise BrokenResourceError("cannot acquire broken resource (original reason): {}".format(self.broken))
+            raise BrokenResourceError(f"cannot acquire broken resource (original reason): {self.broken}")
         super().acquire(*args, **kwargs)
         self.poll()
 
     def release(self, *args, **kwargs):  # pylint: disable=arguments-differ
         if self.broken:
-            raise BrokenResourceError("cannot release broken resource (original reason): {}".format(self.broken))
+            raise BrokenResourceError(f"cannot release broken resource (original reason): {self.broken}")
         super().release(*args, **kwargs)
         self.poll()
 
@@ -191,8 +193,12 @@ class SerialPortExport(ResourceExport):
         self.port = None
         self.ser2net_bin = shutil.which("ser2net")
         if self.ser2net_bin is None:
-            warnings.warn("ser2net binary not found, falling back to /usr/bin/ser2net")
-            self.ser2net_bin = "/usr/bin/ser2net"
+            if os.path.isfile("/usr/sbin/ser2net"):
+                self.ser2net_bin = "/usr/sbin/ser2net"
+
+            if self.ser2net_bin is None:
+                warnings.warn("ser2net binary not found, falling back to /usr/bin/ser2net")
+                self.ser2net_bin = "/usr/bin/ser2net"
 
     def __del__(self):
         if self.child is not None:
@@ -225,28 +231,27 @@ class SerialPortExport(ResourceExport):
         _, _, version = str(subprocess.check_output([self.ser2net_bin,'-v'])).split(' ')
         major_version = version.split('.')[0]
         if int(major_version) >= 4:
-            self.child = subprocess.Popen([
+            cmd = [
                 self.ser2net_bin,
                 '-d',
                 '-n',
-                '-Y', 'connection: &con01#  accepter: telnet(rfc2217,mode=server),{}'.format(self.port),
-                '-Y', '  connector: serialdev(nouucplock=true),{},{}n81,local'.format(start_params['path'], start_params['speed'],
-                ),
-            ])
+                '-Y', f'connection: &con01#  accepter: telnet(rfc2217,mode=server),{self.port}',
+                '-Y', f'  connector: serialdev(nouucplock=true),{start_params["path"]},{self.local.speed}n81,local',  # pylint: disable=line-too-long
+            ]
         else:
-            self.child = subprocess.Popen([
+            cmd = [
                 self.ser2net_bin,
                 '-d',
                 '-n',
                 '-u',
                 '-C',
-                '{}:telnet:0:{}:{} NONE 8DATABITS 1STOPBIT LOCAL'.format(
-                    self.port, start_params['path'], start_params['speed']
-                ),
-            ])
+                f'{self.port}:telnet:0:{start_params["path"]}:{self.local.speed} NONE 8DATABITS 1STOPBIT LOCAL',  # pylint: disable=line-too-long
+            ]
+        self.logger.info("Starting ser2net with: %s", " ".join(cmd))
+        self.child = subprocess.Popen(cmd)
         try:
             self.child.wait(timeout=0.5)
-            raise ExporterError("ser2net for {} exited immediately".format(start_params['path']))
+            raise ExporterError(f"ser2net for {start_params['path']} exited immediately")
         except subprocess.TimeoutExpired:
             # good, ser2net didn't exit immediately
             pass
@@ -274,25 +279,26 @@ exports["USBSerialPort"] = SerialPortExport
 exports["RawSerialPort"] = SerialPortExport
 
 @attr.s(eq=False)
-class USBEthernetExport(ResourceExport):
-    """ResourceExport for a USB ethernet interface"""
+class USBNetworkExport(ResourceExport):
+    """ResourceExport for a USB network interface"""
 
     def __attrs_post_init__(self):
         super().__attrs_post_init__()
-        from ..resource.udev import USBEthernetInterface
-        self.data['cls'] = "EthernetInterface"
-        self.local = USBEthernetInterface(target=None, name=None, **self.local_params)
+        from ..resource.udev import USBNetworkInterface
+        self.data['cls'] = "RemoteNetworkInterface"
+        self.local = USBNetworkInterface(target=None, name=None, **self.local_params)
 
     def _get_params(self):
         """Helper function to return parameters"""
         return {
+            'host': self.host,
             'ifname': self.local.ifname,
             'extra': {
                 'state': self.local.if_state,
             }
         }
 
-exports["USBEthernetInterface"] = USBEthernetExport
+exports["USBNetworkInterface"] = USBNetworkExport
 
 @attr.s(eq=False)
 class USBGenericExport(ResourceExport):
@@ -301,7 +307,7 @@ class USBGenericExport(ResourceExport):
     def __attrs_post_init__(self):
         super().__attrs_post_init__()
         local_cls_name = self.cls
-        self.data['cls'] = "Network{}".format(self.cls)
+        self.data['cls'] = f"Network{self.cls}"
         from ..resource import udev
         local_cls = getattr(udev, local_cls_name)
         self.local = local_cls(target=None, name=None, **self.local_params)
@@ -375,6 +381,26 @@ class USBSDWireExport(USBGenericExport):
             'control_serial': self.local.control_serial,
         }
 
+@attr.s(eq=False)
+class USBAudioInputExport(USBGenericExport):
+    """ResourceExport for ports on switchable USB hubs"""
+
+    def __attrs_post_init__(self):
+        super().__attrs_post_init__()
+
+    def _get_params(self):
+        """Helper function to return parameters"""
+        return {
+            'host': self.host,
+            'busnum': self.local.busnum,
+            'devnum': self.local.devnum,
+            'path': self.local.path,
+            'vendor_id': self.local.vendor_id,
+            'model_id': self.local.model_id,
+            'index': self.local.index,
+            'alsa_name': self.local.alsa_name,
+        }
+
 @attr.s(eq=False)
 class SiSPMPowerPortExport(USBGenericExport):
     """ResourceExport for ports on GEMBRID switches"""
@@ -432,6 +458,35 @@ class USBDeditecRelaisExport(USBGenericExport):
             'index': self.local.index,
         }
 
+@attr.s(eq=False)
+class USBHIDRelayExport(USBGenericExport):
+    """ResourceExport for outputs on simple USB HID relays"""
+
+    def __attrs_post_init__(self):
+        super().__attrs_post_init__()
+
+    def _get_params(self):
+        """Helper function to return parameters"""
+        return {
+            'host': self.host,
+            'busnum': self.local.busnum,
+            'devnum': self.local.devnum,
+            'path': self.local.path,
+            'vendor_id': self.local.vendor_id,
+            'model_id': self.local.model_id,
+            'index': self.local.index,
+        }
+
+@attr.s(eq=False)
+class USBFlashableExport(USBGenericExport):
+    """ResourceExport for Flashable USB devices"""
+    def __attrs_post_init__(self):
+        super().__attrs_post_init__()
+
+    def _get_params(self):
+        p = super()._get_params()
+        p['devnode'] = self.local.devnode
+        return p
 
 exports["AndroidFastboot"] = USBGenericExport
 exports["IMXUSBLoader"] = USBGenericExport
@@ -442,14 +497,42 @@ exports["SigrokUSBDevice"] = USBSigrokExport
 exports["SigrokUSBSerialDevice"] = USBSigrokExport
 exports["USBSDMuxDevice"] = USBSDMuxExport
 exports["USBSDWireDevice"] = USBSDWireExport
+exports["USBDebugger"] = USBGenericExport
 
 exports["USBMassStorage"] = USBGenericExport
 exports["USBVideo"] = USBGenericExport
+exports["USBAudioInput"] = USBAudioInputExport
 exports["USBTMC"] = USBGenericExport
 exports["SiSPMPowerPort"] = SiSPMPowerPortExport
 exports["USBPowerPort"] = USBPowerPortExport
 exports["DeditecRelais8"] = USBDeditecRelaisExport
+exports["HIDRelay"] = USBHIDRelayExport
+exports["USBFlashableDevice"] = USBFlashableExport
+exports["LXAUSBMux"] = USBGenericExport
 
+@attr.s(eq=False)
+class ProviderGenericExport(ResourceExport):
+    """ResourceExport for Resources derived from BaseProvider"""
+
+    def __attrs_post_init__(self):
+        super().__attrs_post_init__()
+        local_cls_name = self.cls
+        self.data['cls'] = f"Remote{self.cls}"
+        from ..resource import provider
+        local_cls = getattr(provider, local_cls_name)
+        self.local = local_cls(target=None, name=None, **self.local_params)
+
+    def _get_params(self):
+        """Helper function to return parameters"""
+        return {
+            'host': self.host,
+            'internal': self.local.internal,
+            'external': self.local.external,
+        }
+
+exports["TFTPProvider"] = ProviderGenericExport
+exports["NFSProvider"] = ProviderGenericExport
+exports["HTTPProvider"] = ProviderGenericExport
 
 @attr.s
 class EthernetPortExport(ResourceExport):
@@ -479,7 +562,7 @@ class GPIOGenericExport(ResourceExport):
     def __attrs_post_init__(self):
         super().__attrs_post_init__()
         local_cls_name = self.cls
-        self.data['cls'] = "Network{}".format(self.cls)
+        self.data['cls'] = f"Network{self.cls}"
         from ..resource import base
         local_cls = getattr(base, local_cls_name)
         self.local = local_cls(target=None, name=None, **self.local_params)
@@ -518,6 +601,20 @@ class NetworkServiceExport(ResourceExport):
 
 exports["NetworkService"] = NetworkServiceExport
 
+@attr.s
+class HTTPVideoStreamExport(ResourceExport):
+    """ResourceExport for an HTTPVideoStream"""
+    def __attrs_post_init__(self):
+        super().__attrs_post_init__()
+        from ..resource.httpvideostream import HTTPVideoStream
+        self.data['cls'] = "HTTPVideoStream"
+        self.local = HTTPVideoStream(target=None, name=None, **self.local_params)
+
+    def _get_params(self):
+        return self.local_params
+
+exports["HTTPVideoStream"] = HTTPVideoStreamExport
+
 @attr.s(eq=False)
 class LXAIOBusNodeExport(ResourceExport):
     """ResourceExport for LXAIOBusNode devices accessed via the HTTP API"""
@@ -525,7 +622,7 @@ class LXAIOBusNodeExport(ResourceExport):
     def __attrs_post_init__(self):
         super().__attrs_post_init__()
         local_cls_name = self.cls
-        self.data['cls'] = "Network{}".format(self.cls)
+        self.data['cls'] = f"Network{self.cls}"
         from ..resource import lxaiobus
         local_cls = getattr(lxaiobus, local_cls_name)
         self.local = local_cls(target=None, name=None, **self.local_params)
@@ -552,8 +649,7 @@ class ExporterSession(ApplicationSession):
         self.groups = {}
 
         enable_tcp_nodelay(self)
-        self.join(self.config.realm, authmethods=["ticket"], authid="exporter/{}".format(
-            self.name))
+        self.join(self.config.realm, authmethods=["ticket"], authid=f"exporter/{self.name}")
 
     def onChallenge(self, challenge):
         """Function invoked on received challege, returns just a dummy ticket
@@ -567,10 +663,10 @@ class ExporterSession(ApplicationSession):
         """
         print(details)
 
-        prefix = 'org.labgrid.exporter.{}'.format(self.name)
-        await self.register(self.acquire, '{}.acquire'.format(prefix))
-        await self.register(self.release, '{}.release'.format(prefix))
-        await self.register(self.version, '{}.version'.format(prefix))
+        prefix = f'org.labgrid.exporter.{self.name}'
+        await self.register(self.acquire, f'{prefix}.acquire')
+        await self.register(self.release, f'{prefix}.release')
+        await self.register(self.version, f'{prefix}.version')
 
         try:
             resource_config = ResourceConfig(self.config.extra['resources'])
@@ -640,7 +736,7 @@ class ExporterSession(ApplicationSession):
                 try:
                     changed = resource.poll()
                 except Exception:  # pylint: disable=broad-except
-                    print("Exception while polling {}".format(resource), file=sys.stderr)
+                    print(f"Exception while polling {resource}", file=sys.stderr)
                     traceback.print_exc()
                     continue
                 if changed:
@@ -660,14 +756,13 @@ class ExporterSession(ApplicationSession):
                 traceback.print_exc()
             age = time.monotonic() - self.checkpoint
             if age > 300:
-                print("missed checkpoint, exiting (last was {} seconds ago)".format(age))
+                print(f"missed checkpoint, exiting (last was {age} seconds ago)")
                 self.disconnect()
 
     async def add_resource(self, group_name, resource_name, cls, params):
         """Add a resource to the exporter and update status on the coordinator"""
         print(
-            "add resource {}/{}: {}/{}".
-            format(group_name, resource_name, cls, params)
+            f"add resource {group_name}/{resource_name}: {cls}/{params}"
         )
         group = self.groups.setdefault(group_name, {})
         assert resource_name not in group
@@ -756,11 +851,11 @@ def main():
     crossbar_url = args.crossbar
     crossbar_realm = os.environ.get("LG_CROSSBAR_REALM", "realm1")
 
-    print("crossbar URL: {}".format(crossbar_url))
-    print("crossbar realm: {}".format(crossbar_realm))
-    print("exporter name: {}".format(extra['name']))
-    print("exporter hostname: {}".format(extra['hostname']))
-    print("resource config file: {}".format(extra['resources']))
+    print(f"crossbar URL: {crossbar_url}")
+    print(f"crossbar realm: {crossbar_realm}")
+    print(f"exporter name: {extra['name']}")
+    print(f"exporter hostname: {extra['hostname']}")
+    print(f"resource config file: {extra['resources']}")
 
     extra['loop'] = loop = asyncio.get_event_loop()
     if args.debug:
