@@ -44,7 +44,7 @@ class Strategy(Driver):  # reuse driver handling
         raise NotImplementedError("Strategies do not support clients")
 
     def transition(self, status):
-        raise NotImplementedError("Strategy.transition() is not implemented for {}".format(self.__class__.__name__))
+        raise NotImplementedError(f"Strategy.transition() is not implemented for {self.__class__.__name__}")
 
     def force(self, status):
-        raise NotImplementedError("Strategy.force() is not implemented for {}".format(self.__class__.__name__))
+        raise NotImplementedError(f"Strategy.force() is not implemented for {self.__class__.__name__}")
